#pragma once
#include "B4RDefines.h"
#include <RCSwitch.h>

//~version: 1.01
//~Author: JDK
namespace B4R {
	//~shortname: RCSwitch
	//~Event: DataAvailable()
	//Operate 433/315 MHz radio controlled devices. This will most likely work with all popular low cost power outlet sockets.
	//If yours doesn't work, you might need to adjust the pulse length.
	//Supported chipsets:
	//SC5262 / SC5272
	//HX2262 / HX2272
	//PT2262 / PT2272
	//EV1527 / RT1527 / FP1527 / HS1527
	//Intertechno outlets
	//HT6P20X
	//Example: <code>
	//Private rcs As RCSwitch
	//</code>
	class B4RRCSwitch {
		private:      
			uint8_t backend[sizeof(RCSwitch)];
			RCSwitch* rcs;
			PollerNode pnode;
			SubVoidVoid DataAvailableSub;
			static void looper(void* b);
		public:
			//Initializes the library.
			//Must be the first function called
			//Example: <code>
			//rcs.Initialize)
			//</code>
			void Initialize();

			//Enable a receiver connected to Receiverpin
			//Only pins with interrupt works
			//Example: <code>
			//rcs.enableReceive("rcs_Available", rcs.WEMOS_Receivepin_8)
			//</code>
			void EnableReceive(SubVoidVoid DataAvailableSub, int ReceiverPin);

			//After reading the received data, call ResetAvailable
			//Data received between event DataAvailableSub and ResetAvailable are ignored.
			//Example: <code>
			//rcs.ResetAvailable
			//</code>
			void ResetAvailable();

			//Returns the received code
			//Example: <code>
			//Log("Code: ", rcs.ReceivedCode)
			//</code>
			ULong getReceivedCode();

			//Returns the received bit length
			//Example: <code>
			//Log("Bit length: ", rcs.ReceivedBitLength)
			//</code>
			UInt getReceivedBitLength();

			//Returns the received delay
			//Example: <code>
			//Log("Delay: ", rcs.ReceivedDelay)
			//</code>
			UInt getReceivedDelay();

			//Returns the received protocol
			//Example: <code>
			//Log("Protocol: ", rcs.ReceivedProtocol)
			//</code>
			UInt getReceivedProtocol();

			//Enable a transmitter connected to Transmitterpin
			//Example: <code>
			//rcs.EnableTransmit(7)
			//</code>
			void EnableTransmit(int TransmitterPin);
			
			//Sends a code
			//Example: <code>
			//rcs.Send(123456, 24)
			//</code>
			void Send(ULong Code, UInt BitLength);
			
			//Sets the protocol to transmit
			//Default is 1
			//Example: <code>
			//rcs.Protocol = 1
			//</code>
			void setProtocol(int Protocol);
 			
			//Sets the pulselength
			//Example: <code>
			//rcs.setPulseLength=320
			//</code>
			void setPulseLength(int PulseLength);
			
			//Sets the number of times the code will be sent
			//Default is 4 times
			//Example: <code>
			//rcs.RepeatTransmit = rcs.Repeat_1_time
			//</code>
			void setRepeatTransmit(int RepeatTransmit);

			
			//ENUMS
			
			//Example: <code>
			//rcs.enableReceive("rcs_Available", rcs.UNO_Receivepin_2)
			//</code>
			static const int UNO_Receivepin_2	= 0;

			//Example: <code>
			//rcs.enableReceive("rcs_Available", rcs.UNO_Receivepin_3)
			//</code>
			static const int UNO_Receivepin_3	= 1;

			//Example: <code>
			//rcs.enableReceive("rcs_Available", rcs.MEGA_Receivepin__2)
			//</code>
			static const int MEGA_Receivepin__2	= 0;

			//Example: <code>
			//rcs.enableReceive("rcs_Available", rcs.MEGA_Receivepin__3)
			//</code>
			static const int MEGA_Receivepin__3	= 1;

			//Example: <code>
			//rcs.enableReceive("rcs_Available", rcs.MEGA_Receivepin_18)
			//</code>
			static const int MEGA_Receivepin_18	= 5;

			//Example: <code>
			//rcs.enableReceive("rcs_Available", rcs.MEGA_Receivepin_19)
			//</code>
			static const int MEGA_Receivepin_19	= 4;

			//Example: <code>
			//rcs.enableReceive("rcs_Available", rcs.MEGA_Receivepin_20)
			//</code>
			static const int MEGA_Receivepin_20	= 3;

			//Example: <code>
			//rcs.enableReceive("rcs_Available", rcs.MEGA_Receivepin_21)
			//</code>
			static const int MEGA_Receivepin_21	= 2;

			//Example: <code>
			//rcs.enableReceive("rcs_Available", rcs.WEMOS_Receivepin_1)
			//</code>
			static const int WEMOS_Receivepin_1	= 5;

			//Example: <code>
			//rcs.enableReceive("rcs_Available", rcs.WEMOS_Receivepin_2)
			//</code>
			static const int WEMOS_Receivepin_2	= 4;

			//Example: <code>
			//rcs.enableReceive("rcs_Available", rcs.WEMOS_Receivepin_3)
			//</code>
			static const int WEMOS_Receivepin_3	= 0;

			//Example: <code>
			//rcs.enableReceive("rcs_Available", rcs.WEMOS_Receivepin_4)
			//</code>
			static const int WEMOS_Receivepin_4	= 2;

			//Example: <code>
			//rcs.enableReceive("rcs_Available", rcs.WEMOS_Receivepin_5)
			//</code>
			static const int WEMOS_Receivepin_5	= 14;

			//Example: <code>
			//rcs.enableReceive("rcs_Available", rcs.WEMOS_Receivepin_6)
			//</code>
			static const int WEMOS_Receivepin_6	= 12;

			//Example: <code>
			//rcs.enableReceive("rcs_Available", rcs.WEMOS_Receivepin_7)
			//</code>
			static const int WEMOS_Receivepin_7	= 13;

			//Example: <code>
			//rcs.enableReceive("rcs_Available", rcs.WEMOS_Receivepin_8)
			//</code>
			static const int WEMOS_Receivepin_8	= 15;

			
			//Example: <code>
			//rcs.RepeatTransmit = rcs.Repeat_1_time
			//</code>
			static const Int Repeat_1_time	= 4;
			
			//Example: <code>
			//rcs.RepeatTransmit = rcs.Repeat_2_times
			//</code>
			static const Int Repeat_2_times	= 6;
			
			//Example: <code>
			//rcs.RepeatTransmit = rcs.Repeat_3_times
			//</code>
			static const Int Repeat_3_times	= 8;

			//Example: <code>
			//rcs.RepeatTransmit = rcs.Repeat_4_times
			//</code>
			static const Int Repeat_4_times	= 12;

			//Example: <code>
			//rcs.RepeatTransmit = rcs.Repeat_5_times
			//</code>
			static const Int Repeat_5_times	= 14;

			//Example: <code>
			//rcs.RepeatTransmit = rcs.Repeat_6_times
			//</code>
			static const Int Repeat_6_times	= 18;

			//Example: <code>
			//rcs.RepeatTransmit = rcs.Repeat_7_times
			//</code>
			static const Int Repeat_7_times	= 20;

			//Example: <code>
			//rcs.RepeatTransmit = rcs.Repeat_8_times
			//</code>
			static const Int Repeat_8_times	= 25;

			//Example: <code>
			//rcs.RepeatTransmit = rcs.Repeat_9_times
			//</code>
			static const Int Repeat_9_times	= 26;

			//Example: <code>
			//rcs.RepeatTransmit = rcs.Repeat_10_times
			//</code>
			static const Int Repeat10_times	= 30;
	};
}
